import os
import logging
import json

from typing import Union, Optional

from .api import event, task_event
from .appliance import min_ppa_version

logger = logging.getLogger(__name__)


def _load_json_file(path) -> dict:
    with open(path, "r") as open_file:
        contents = "".join(open_file.readlines())
    try:
        return json.loads(contents)
    except json.JSONDecodeError as e:
        logger.info(f"Failed to load JSON file {path}")
        raise e


def manifest() -> dict:
    """ Get the task manifest file in dictionary format. The manifest dictionary will
    contain the following keys:

        - `labels`: a dictionary of labels from the task Dockerfile.
        - `groups`: a list of any specified groups in the Dockerfile.
        - `version`: the manifest file format version.

    :returns: A dictionary containing task manifest data.

    Example:

        .. code-block:: python

            manifest = task.manifest()
            groups = manifest.get("groups")
            if groups:
                ui.output_info(f"Only {len(groups)} groups may run this task")
            ui.output_info(
                f"The task Dockerfile has the following labels: {manifest.get('labels')}"
            )
    """
    return _load_json_file("/.ppa/manifest.json")


@min_ppa_version("2.4.0")
def payload() -> Optional[dict]:
    """ Get the task payload file in dictionary format. The payload can be supplied via a schedule or API call.

    :returns: If a payload file exists, a dictionary containing the payload data will be
        returned. If no payload exists, this function will return None.

    Example:

        .. code-block:: python

            payload = task.payload()
            if "database_name" not in payload:
                database_name = ui.input_text("Provide a database name")
    """
    try:
        return _load_json_file("/.ppa/payload.json")
    except FileNotFoundError:
        return None


@min_ppa_version("2.4.0")
def trigger() -> str:
    """ Return the trigger type of the task as a string.

    The trigger will be one of:

        `ui`: started by a user from the PPA interface.
        `api`: started from a call to the tasks API, either remotely or by another task.
        `job`: started by a schedule.
    """
    return os.environ["PPA_TRIGGER"]


def user() -> str:
    """ Get the login username of the user running the task. If the user logged in
    with an Active Directory user account, the domain prefix will be included.

    Example:

        .. code-block:: python

            ppa_username = task.user()
            try:
                ppa_username = ppa_username.split("\\\\")[1]
            except IndexError:
                # not a domain account
                pass

    """
    return os.environ["PPA_USER"]


@task_event()
def user_password(text: str, element_id: Optional[str] = None) -> Union[str, None]:
    """ Request the login password for the user running the task. This
    function will ask the user for permission before the password is
    delivered to the task.

    This function can be used to perform passthrough authentication against
    servers and applications. It can also be used to verify the user's identity
    against a target system before operations are performed.

    :param text: The text to display to the user when requesting their login password.
    :param element_id: The ID of the UI element. See :ref:`getting_started:Input & Output IDs`.

    :returns: If the user consents, returns the user's login password as a string. If
        the user does not consent, returns None. If the user has logged in to PPA using
        SAML, the PPA appliance will not have encrypted and stored the user's password and
        None will be returned.


    Example where the user login credentials are validated against another application:

        .. code-block:: python

            ppa_username = task.user()
            try:
                ppa_username = ppa_username.split("\\\\")[1]
            except IndexError:
                # not a domain account
                pass

            ppa_password = task.user_password("Application identity validation")
            if ppa_password:
                # hypothetical authentication function
                my_application.authenticate(
                    username=ppa_username,
                    password=ppa_password
                )
            else:
                ui.output_error("Validation attempt rejected")

    """
    return event("user/password", text, id=element_id)
