import logging
from typing import List, Union, Optional
from collections import namedtuple

from ppa import appliance, ui

logger = logging.getLogger(__name__)

_vault_fields = ["name", "url", "driver", "expires_at"]


class Vault(namedtuple("vault", _vault_fields)):  # type: ignore
    pass


class NoVaultsFound(Exception):
    pass


def vaults(reason: str) -> List[Vault]:
    """ Return all configured vaults.
    """
    return [
        Vault(*[api.get(item) for item in _vault_fields])  # type: ignore
        for api in appliance.apis(reason=reason).get("data")
    ]


def _vaults_by_driver(driver: str, reason: str) -> List[Vault]:
    """ Return a list of all configured credentials vaults using the
    supplied driver.
    """
    return [v for v in vaults(reason) if v.driver == driver]  # type: ignore


def hashicorp_vaults(reason: Optional[Union[str, None]] = None) -> List[Vault]:
    """ Get all configured Hashicorp Vaults.

    :param reason: A justification for the API call. This will be
        present in the API request event log. Defaults to 'Getting
        all configured Hashicorp Vaults'.

    :raises exceptions.NoVaultsFound: if there are no configured Hashicorp Vaults.

    :returns: A list of :ref:`vaults:Vault Tuple` objects.

    Example:

        .. code-block:: python

            vaults = hashicorp_vaults(
                reason="Task requires access to Hashicorp Vault"
            )

    """
    reason = reason or f"Getting all configured Hashicorp Vaults"
    vaults = _vaults_by_driver("hashicorp-vault", reason=reason)
    if not vaults:
        raise NoVaultsFound("No Hashicorp Vaults found in PPA Vaults configuration.")
    return vaults


def pxm_vaults(reason: Optional[Union[str, None]] = None) -> List[Vault]:
    """ Get all configured PxM Appliances vaults.

    :param reason: A justification for the API call. This will be
        present in the API request event log. Defaults to 'Getting
        all configured PxM Appliances'.

    :raises exceptions.NoVaultsFound: if there are no configured PxM Appliance vaults.

    :returns: A list of :ref:`vaults:Vault Tuple` objects.

    Example:

        .. code-block:: python

            vaults = pxm_vaults(
                reason="Task requires access to PxM Appliance vault"
            )

    """
    reason = reason or f"Getting all configured PxM Appliances"
    vaults = _vaults_by_driver("osirium-pxm-rest", reason=reason)
    if not vaults:
        raise NoVaultsFound("No PxM Appliances found in PPA Vaults configuration.")
    return vaults


def pxm_vault_by_name(name: str, reason: Optional[Union[str, None]] = None) -> Vault:
    """ Get a configured PxM Appliance vault with a specific name.

    :param name: The name of the PxM Appliance vault.
    :param reason: A justification for the API call. This will
        be present in the API request event log. Defaults to
        f"Getting PxM Appliance with name {name}".

    :raises exceptions.NoVaultsFound: if no configured PxM Appliance vault
        exists with the supplied name.

    :returns: A single :ref:`vaults:Vault Tuple`.

    Example:

        .. code-block:: python

            name = "corporate.pxm.net"
            vaults = pxm_vault_by_name(
                name,
                reason=f"Task requires access to PxM Appliance vault {name}"
            )
    """
    reason = reason or f"Getting PxM Appliance with name {name}"
    try:
        return next((pxm for pxm in pxm_vaults(reason=reason) if pxm.name == name))  # type: ignore
    except StopIteration:
        raise NoVaultsFound(f"No PxM Appliances found with name {name} in PPA Vaults configuration.")


def hashicorp_vault_by_name(name: str, reason: Optional[Union[str, None]] = None) -> Vault:
    """ Get a configured Hashicorp Vault with a specific name.

    :param name: The name of the Hashicorp Vault.
    :param reason: A justification for the API call. This will
        be present in the API request event log. Defaults to
        f"Getting Hashicorp Vault with name {name}".

    :raises exceptions.NoVaultsFound: if no configured Hashicorp Vault
        exists with the supplied name.

    :returns: A single :ref:`vaults:Vault Tuple`.

    Example:

        .. code-block:: python

            name = "corporate.hashicorp.vault"
            vaults = hashicorp_vault_by_name(
                name,
                reason=f"Task requires access to Hashicorp Vault {name}"
            )
    """
    reason = reason or f"Getting Hashicorp Vault with name {name}"
    try:
        return next((hv for hv in hashicorp_vaults(reason=reason) if hv.name == name))  # type: ignore
    except StopIteration:
        raise NoVaultsFound(f"No Hashicorp vaults found with name {name} in PPA Vaults configuration.")


def select_vault(vaults: List[Vault]) -> Vault:
    """
    Select a single vault from a list of :ref:`vaults:Vault Tuple`
    records.

    You can wrap this function around :py:func:`vaults.hashicorp_vaults`
    and :py:func:`vaults.pxm_vaults`, automatically prompting the user to
    select a single vault if more than one is returned.

    :param vaults: A list of :ref:`vaults:Vault Tuple` objects.

    :raises exceptions.NoVaultsFound: if the supplied vaults list is empty.

    :returns: A single :ref:`vaults:Vault Tuple`.

    Example:

        .. code-block:: python

            vault = select_vault(
                pxm_vaults(
                    reason="Task requires access to PxM Appliance vault"
                )
            )
    """
    if len(vaults) > 1:
        while True:
            indices = ui.input_table(
                "Select a vault to use",
                header=["Name", "URL", "Driver", "Expiry"],
                data=[[getattr(v, item) or "N/A" for item in _vault_fields] for v in vaults],
                element_id="select_vault",
            )
            if any([not indices, len(indices) > 1]):
                ui.output_error("Please choose a single entry.")
            else:
                break
        return vaults[indices[0]]
    try:
        return vaults[0]
    except IndexError:
        raise NoVaultsFound("An empty vaults list was supplied.")
